unit ConnectForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, dbisamtb, ShlObj, ActiveX;

type
  TfmConnect = class(TForm)
    edUserName: TEdit;
    edPassword: TEdit;
    laUserName: TLabel;
    laPassword: TLabel;
    paButtons: TPanel;
    buOk: TButton;
    buCancel: TButton;
    Bevel1: TBevel;
    laHost: TLabel;
    edHost: TEdit;
    rgSessionType: TRadioGroup;
    laLocalDir: TLabel;
    edLocalDir: TEdit;
    Bevel2: TBevel;
    Bevel3: TBevel;
    buBrowse: TButton;
    procedure buBrowseClick(Sender: TObject);
    procedure rgSessionTypeClick(Sender: TObject);
  private
    procedure EnableSession(Local: boolean);
  end;

function ShowConnectForm(Database: TDBISAMDatabase): boolean;

implementation

{$R *.DFM}

function ShowConnectForm(Database: TDBISAMDatabase): boolean;
var
  fmConnect: TfmConnect;
begin
  Result := False;
  if not Assigned(Database) or not Assigned(Database.Session) then Exit;
  fmConnect := TfmConnect.Create(nil);
  try
    if Database.Session.SessionType = stLocal
      then fmConnect.rgSessionType.ItemIndex := 0
      else fmConnect.rgSessionType.ItemIndex := 1;
    fmConnect.edLocalDir.Text := Database.Directory;
    fmConnect.edHost.Text := Database.Session.RemoteHost;
    fmConnect.edUserName.Text := Database.Session.RemoteUser;
    fmConnect.edPassword.Text := Database.Session.RemotePassword;
    Result := fmConnect.ShowModal() = mrOk;
    if Result then begin
      Database.Session.Active := False;
      if fmConnect.rgSessionType.ItemIndex = 0
        then Database.Session.SessionType := stLocal
        else Database.Session.SessionType := stRemote;
      Database.Directory := fmConnect.edLocalDir.Text;
      Database.Session.RemoteHost := fmConnect.edHost.Text;
      Database.Session.RemoteUser := fmConnect.edUserName.Text;
      Database.Session.RemotePassword := fmConnect.edPassword.Text;
      Database.Session.Active := True;
    end;
  finally
    fmConnect.Free();
  end;
end;

procedure TfmConnect.buBrowseClick(Sender: TObject);

  function NormalDir (const DirName : string) : string;
  begin
    Result := DirName;
    if Result = EmptyStr then exit;
    if Result[length(Result)]<>'\' then Result := Result + '\';
  end;

var
  BrowseInfo: TBrowseInfo;
  ItemIDList: PItemIDList;
  TempPath, Buffer: array [0..MAX_PATH] of Char;
  Result: boolean;
begin
  ItemIDList := nil;
  try
    BrowseInfo.hwndOwner := Handle;
    BrowseInfo.pidlRoot := nil;
    BrowseInfo.pszDisplayName := Buffer;
    BrowseInfo.lpszTitle := 'Select directory';
    BrowseInfo.ulFlags := 0;
    BrowseInfo.lpfn := nil;
    BrowseInfo.LPARAM := 0;
    BrowseInfo.iImage := 0;

    ItemIDList := SHBrowseForFolder(BrowseInfo);
    Result := ItemIDList <> nil;
    if Result then begin
      SHGetPathFromIDList(ItemIDList, TempPath);
      edLocalDir.Text := StrPas(TempPath);
      if (edLocalDir.Text <> '') and (edLocalDir.Text[Length(edLocalDir.Text)] <> '\')
        then edLocalDir.Text := edLocalDir.Text + '\';
    end;
  finally
    CoTaskMemFree(ItemIDList);
    CoTaskMemFree(BrowseInfo.pidlRoot);
  end;
end;

procedure TfmConnect.EnableSession(Local: boolean);
begin
  laLocalDir.Enabled := Local;
  edLocalDir.Enabled := Local;
  buBrowse.Enabled := Local;

  laHost.Enabled := not Local;
  edHost.Enabled := not Local;
  laUserName.Enabled := not Local;
  edUserName.Enabled := not Local;
  laPassword.Enabled := not Local;
  edPassword.Enabled := not Local;
end;

procedure TfmConnect.rgSessionTypeClick(Sender: TObject);
begin
  EnableSession(rgSessionType.ItemIndex = 0);
end;

end.
